import java.io.*;
import java.util.*;

/**
 * Organiza y almacena Proyectos.
 */
public class Organizador {
	public Organizador() {
		asignatura = new String("desconocido");
		listadoAlumnos = new Vector();
		proyectos = new Vector();
	}

	public void setAsignatura(String asignatura) {
		this.asignatura = new String(asignatura);
	}

	public String getAsignatura() {
		return new String(asignatura);
	}

	public void setListadoAlumnos(String archivo) {
		String s = new String();
		try {
			BufferedReader br = 
				new BufferedReader(
					new FileReader(archivo));

			// Aqui se aprovecha el hecho de que en el archivo con
			// los nombres, los nombres van en las lineas pares.
			while((s = br.readLine()) != null) {
				listadoAlumnos.addElement(br.readLine());
			}

			br.close();
		} catch(Throwable excepcion) {
			System.out.println(excepcion);
		}
	}

	public String getListadoAlumnos() {
		String s = new String();
		for(int i = 0; i < (listadoAlumnos.size() - 1); i++)
			s = s + listadoAlumnos.elementAt(i) + "\n";
		s = s + listadoAlumnos.elementAt(listadoAlumnos.size() - 1);
		return s;
	}

	public void setProyectos(String archivo) {
		String temp = new String();
		Proyecto proyecto = new Proyecto();
		try {
			BufferedReader br = 
				new BufferedReader(
					new FileReader(archivo));

			while((temp = br.readLine()) != null) {
				if (temp.equalsIgnoreCase("<proyecto>")) {
					proyecto.setProyectoDesdeArchivo(br);
					proyectos.addElement(new Proyecto(proyecto));
				}
			}

			br.close();
		} catch(Throwable excepcion) {
			System.out.println(excepcion);
		}
	}

	public String getProyectos() {
		String s = new String();
		for(int i = 0; i < (proyectos.size() - 1); i++)
			s = s + proyectos.elementAt(i) + "\n";
		s = s + proyectos.elementAt(proyectos.size() - 1);
		return s;
	}

	public int getNumeroProyectos() {
		return proyectos.size();
	}

	public int getNumeroAlumnos() {
		return listadoAlumnos.size();
	}

	public Proyecto getProyectoEnIndice(int indice) {
		if((indice >= 0) && (indice < proyectos.size()))
			return new Proyecto((Proyecto) proyectos.get(indice));
		else 
			return new Proyecto();
	}

	public String getNombreAlumnoEnProyectoEnIndice(int indice) {
		return (getProyectoEnIndice(indice)).getNombreAlumno();
	}

	public String getNombreProyectoEnProyectoEnIndice(int indice) {
		return (getProyectoEnIndice(indice)).getNombreProyecto();
	}

	public String getDescripcionProblemaEnProyectoEnIndice(int indice) {
		return (getProyectoEnIndice(indice)).getDescripcionProblema();
	}

	public String getSolucionInicialEnProyectoEnIndice(int indice) {
		return (getProyectoEnIndice(indice)).getSolucionInicial();
	}

/**
 * Entrega un listado con los alumnos que a&uacute;n no inscriben su
 * proyecto.
 * @return El listado con los alumnos que no han inscrito su proyecto.
 */
	public String getAlumnosFaltantes() {
		boolean a;
		String s = new String();
		Vector b = new Vector(listadoAlumnos);

		for(int i = 0; i < getNumeroProyectos(); i++) {
			s = getNombreAlumnoEnProyectoEnIndice(i);
			a = b.remove(s);
		}

		s = new String();

		for(int i = 0; i < (b.size() - 1); i++)
			s = s + b.elementAt(i) + "\n";
		s = s + b.elementAt(b.size() - 1);

		return s;
	}
		
	public void eliminarProyectoEnIndice(int indice) {
		if((indice >= 0) && (indice < proyectos.size())) {
			proyectos.removeElementAt(indice);
		}
	}

	public void insertarProyectoEnIndice(Proyecto proyecto, int indice) {
		if((indice >= 0) && (indice <= proyectos.size())) {
			proyectos.insertElementAt(proyecto, indice);
		}
	}

/**
 * Intercambia de lugar un Proyecto con el siguiente.
 * Si el Proyecto es el &uacute;ltimo, se queda donde est&aacute;.
 * @param indice El indice del Proyecto a mover.
 * @return El indice donde qued&oacute; el Proyecto. En caso de que el
 * 		indice que le fue entregado no es v&aacute;lido, entrega -1.
 */
	public int avanzaUnLugarProyectoEnIndice(int indice) {
		if((indice >= 0) && (indice < proyectos.size())) {
			if(indice < (proyectos.size() - 1)) {
				Proyecto proyecto;
				proyecto = new Proyecto((Proyecto) proyectos.remove(indice));
				proyectos.insertElementAt(proyecto, indice + 1);
				return indice + 1;
			}
			else
				return indice;
		}
		else
			return -1;
	}
				
/**
 * Intercambia de lugar un Proyecto con el anterior.
 * Si el Proyecto es el primero, se queda donde est&aacute;.
 * @param indice El indice del Proyecto a mover.
 * @return El indice donde qued&oacute; el Proyecto. En caso de que el
 * 		indice que le fue entregado no es v&aacute;lido, entrega -1.
 */
	public int retrocedeUnLugarProyectoEnIndice(int indice) {
		if((indice >= 0) && (indice < proyectos.size())) {
			if(indice > 0) {
				Proyecto proyecto;
				proyecto = new Proyecto((Proyecto) proyectos.remove(indice));
				proyectos.insertElementAt(proyecto, indice - 1);
				return indice - 1;
			}
			else
				return indice;
		}
		else
			return -1;
	}

/**
 * Agrega un Proyecto y lo coloca al final.
 */
	public void addProyecto(Proyecto proyecto) {
		proyectos.addElement(proyecto);
	}

/**
 * Escribe los proyectos en formato HTML, utilizando un <TT>modeloP</TT> 
 * para la p&aacute;gina en general, y un <TT>modeloT</TT> para los
 * proyectos propiamente tal.
 * @param modeloP El c&oacute;digo HTML que servir&aacute; de base para la
 * p&aacute;gina.
 * @param modeloT El c&oacute;digo HTML que servir&aacute; de base para los
 * proyectos.
 * @return El c&oacute;digo HTML de la p&aacute;gina completa.
 */
	public String toStringHTML(String modeloP, String modeloT) {
		String s = new String(); 
		String temp = new String();
		try {
			BufferedReader brP = 
				new BufferedReader(
					new FileReader(modeloP));

			while((temp = brP.readLine()) != null) {
				if(temp.equalsIgnoreCase("<!--asignatura-->")) {
					s = s + asignatura+ "\n";
				}
				else if(temp.equalsIgnoreCase("<!--alumnos_faltantes-->")) {
					s = s + utilString.reemplazar(getAlumnosFaltantes(), "\n", "<br>") + "\n";
				}
				else if(temp.equalsIgnoreCase("<!--proyectos-->")) {

					BufferedReader brT = 
						new BufferedReader(
							new FileReader(modeloT));

					while((temp = brT.readLine()) != null) {
						if(temp.equalsIgnoreCase("<!--nombre_alumno-->")) {
							s = s + "<B>Nombre</B>" + "\n";
						}
						else if(temp.equalsIgnoreCase("<!--nombre_proyecto-->")) {
						s = s + "<B>Nombre del Proyecto</B>" + "\n";
						}
						else if(temp.equalsIgnoreCase("<!--descripcion_problema-->")) {
						s = s + "<B>Descripci&oacute;n del problema</B>" + "\n";
						}
						else if(temp.equalsIgnoreCase("<!--solucion_inicial-->")) {
							s = s + "<B>Idea inicial de una posible soluci&oacute;n</B>" + "\n";
						}
						else {
							s = s + temp + "\n";
						}
					}
					brT.close();
					for(int i = 0; i < proyectos.size(); i++) {
						s = s + (getProyectoEnIndice(i)).toStringHTML(modeloT) + "\n";
					}
				}
				else {
					s = s + temp + "\n";
				}
			}
			brP.close();
		} catch(Throwable excepcion) {
			System.out.println(excepcion);
			s = "Error";
		}
		return s;
	}


	private String asignatura;
	private Vector listadoAlumnos;
	private Vector proyectos;
}